@@ -54,7 +54,9 @@ class PackageConfig
     end
 
     def guess_native_pkg_config
-      pkg_config = with_config("pkg-config", ENV["PKG_CONFIG"] || "pkg-config")
+      exeext = RbConfig::CONFIG["EXEEXT"]
+      default_pkg_config = ENV["PKG_CONFIG"] || "pkg-config#{exeext}"
+      pkg_config = with_config("pkg-config", default_pkg_config)
       pkg_config = Pathname.new(pkg_config)
       unless pkg_config.absolute?
         found_pkg_config = search_pkg_config_from_path(pkg_config)
@@ -208,10 +210,10 @@ class PackageConfig
   private
   def collect_cflags
     cflags_set = [declaration("Cflags")]
-    cflags_set += required_packages.collect do |package|
+    cflags_set += private_required_packages.collect do |package|
       self.class.new(package, @options).cflags
     end
-    cflags_set += private_required_packages.collect do |package|
+    cflags_set += required_packages.collect do |package|
       self.class.new(package, @options).cflags
     end
     all_cflags = normalize_cflags(Shellwords.split(cflags_set.join(" ")))
@@ -234,10 +236,16 @@ class PackageConfig
       path = path_flag.sub(flag_option, "")
       prefix = ""
       case RUBY_PLATFORM
-      when "x86-mingw32"
-        prefix = Dir.glob("c:/msys{32,64,*}").first
+      when "i386-mingw32"
+        ruby_prefix = RbConfig::CONFIG["prefix"]
+        candidates = Dir.glob("#{ruby_prefix}/msys{32,64,*}")
+        candidates.concat(Dir.glob("c:/msys{32,64,*}"))
+        prefix = candidates.first
       when "x64-mingw32"
-        prefix = Dir.glob("c:/msys{64,*}").first
+        ruby_prefix = RbConfig::CONFIG["prefix"]
+        candidates = Dir.glob("#{ruby_prefix}/msys{64,*}")
+        candidates.concat(Dir.glob("c:/msys{64,*}"))
+        prefix = candidates.first
       end
       if /\A[a-z]:/i === path
         "#{flag_option}#{path}"
@@ -357,8 +365,12 @@ class PackageConfig
     ]
     case RUBY_PLATFORM
     when "x86-mingw32"
+      prefix = RbConfig::CONFIG["prefix"]
+      default_paths.concat(Dir.glob("#{prefix}/msys*/mingw32/lib/pkgconfig"))
       default_paths.concat(Dir.glob("c:/msys*/mingw32/lib/pkgconfig"))
     when "x64-mingw32"
+      prefix = RbConfig::CONFIG["prefix"]
+      default_paths.concat(Dir.glob("#{prefix}/msys*/mingw64/lib/pkgconfig"))
       default_paths.concat(Dir.glob("c:/msys*/mingw64/lib/pkgconfig"))
     end
     libdir = ENV["PKG_CONFIG_LIBDIR"]
@@ -520,7 +532,9 @@ module PKGConfig
         $LDFLAGS += ' ' + dldflags
       end
       $CFLAGS += ' ' + cflags_only_other(pkg)
-      $CXXFLAGS += ' ' + cflags_only_other(pkg)
+      if defined?($CXXFLAGS)
+        $CXXFLAGS += ' ' + cflags_only_other(pkg)
+      end
       $INCFLAGS += ' ' + cflags_only_I(pkg)
     end
     enough_version
